@@ -2,11 +2,19 @@ import math
 from abc import ABC, abstractmethod
 from functools import lru_cache, partial
 from itertools import combinations
-from typing import Callable, List, Optional, Tuple
+from typing import Callable
+from typing import Iterable as IterableType
+from typing import List, Optional, Tuple, Union
 
 import numpy as np
 import pandas as pd
 
+from profiling.utils import (
+    batched_non_object_col_mode,
+    batched_numeric_col_mean,
+    drop_nas_from_batch,
+)
+
 
 @lru_cache(maxsize=256)
 def n_choose_k(n, k):
@@ -32,24 +40,6 @@ comb = np.vectorize(
 )
 
 
-def fast_object_mode(s):
-    """Calculate mode of Series through intermediate string representation instead of hash table
-
-    Parameters
-    ----------
-    s : pd.Series
-        Pandas Series
-
-    Returns
-    -------
-    Modal value
-    """
-    s_string = s.astype(str)
-    _, indices, counts = np.unique(s_string, return_index=True, return_counts=True)
-    index_max = indices[counts.argmax()]
-    return s.iloc[index_max]
-
-
 class Explainer(ABC):
     @abstractmethod
     def explain(self, x):
@@ -108,7 +98,8 @@ class ShapExplainer(Explainer):
     def __init__(
         self,
         func: Callable[[pd.DataFrame], pd.Series],
-        x: pd.DataFrame,
+        x: Union[pd.DataFrame, IterableType[pd.DataFrame]],
+        y: IterableType[pd.Series] = None,
         profile_columns: Optional[List[str]] = None,
     ):
         """Create a Shap Explainer
@@ -122,25 +113,50 @@ class ShapExplainer(Explainer):
         profile_columns : list, optional
             Calculate Shapley values only for a subset of columns, by default all columns will be included
         """
-        if profile_columns is not None:
-            func = reduce_func(func, x=self._create_background_data(x))
+
+        if profile_columns is not None and isinstance(x, pd.DataFrame):
+            func = reduce_func(func, x=self._create_background_data(x, y=y))
             x = x[profile_columns]
 
         self.func = func
-        self.x = self._create_background_data(x)
+        self.x = self._create_background_data(x, y=y)
         self.n_col = len(self.x.columns)
 
-    @staticmethod
-    def _create_background_data(x) -> pd.DataFrame:
+    def _create_background_data(self, x, y=None) -> pd.DataFrame:
         """Create single-row background dataset"""
+        if isinstance(x, pd.DataFrame):
+            return self._create_background_data_from_full_frame(x)
+        else:
+            first = drop_nas_from_batch(next(x), next(y))
+            return self._created_batched_background_data(first=first, x=x, y=y)
+
+    @staticmethod
+    def _created_batched_background_data(
+        first: pd.DataFrame, x: IterableType[pd.DataFrame], y: IterableType[pd.Series]
+    ):
+        background = {}
+        for col in first.columns:
+            if pd.api.types.is_numeric_dtype(first[col]):
+                background[col] = [
+                    batched_numeric_col_mean(x=x, y=y, first=first, col=col)
+                ]
+            else:
+                background[col] = [
+                    batched_non_object_col_mode(x=x, y=y, first=first, col=col)
+                ]
+        x_background = pd.DataFrame(background, columns=first.columns).astype(
+            first.dtypes
+        )
+        return x_background
+
+    @staticmethod
+    def _create_background_data_from_full_frame(x: pd.DataFrame):
         background = {}
         for col in x.columns:
             if pd.api.types.is_numeric_dtype(x[col]):
-                background[col] = [x[col].median()]
-            elif pd.api.types.is_object_dtype(x[col]):
-                background[col] = [fast_object_mode(x[col])]
+                background[col] = [x[col].mean()]
             else:
-                background[col] = [x[col].mode(dropna=False).values[0]]
+                background[col] = [x[col].mode(dropna=True).values[0]]
         x_background = pd.DataFrame(background, columns=x.columns).astype(x.dtypes)
         return x_background
 
